@@ -40,11 +40,12 @@ class Pipeline:
                  target: str = '',
                  name: str = '',
                  version: str = None,
-                 mode: str = 'classification',
-                 objective: str = 'neg_log_loss',
+                 mode: str = None,
+                 objective: str = None,
 
                  # Data Processing
-                 num_cols: list = None,
+                 int_cols: list = None,
+                 float_cols: list = None,
                  date_cols: list = None,
                  cat_cols: list = None,
                  missing_values: str = 'zero',
@@ -70,7 +71,7 @@ class Pipeline:
                  # Initial Modelling
                  standardize: bool = False,
                  shuffle: bool = True,
-                 cv_splits: int = 3,
+                 cv_splits: int = 10,
                  store_models: bool = False,
 
                  # Grid Search
@@ -83,13 +84,14 @@ class Pipeline:
                  stacking: bool = False,
 
                  # Production
-                 custom_function: str = None,
+                 preprocess_function: str = None,
 
                  # Flags
                  plot_eda: bool = True,
                  process_data: bool = True,
                  document_results: bool = True,
-                 verbose: int = 1):
+                 verbose: int = 1,
+                 no_dirs: bool = False):
         """
         Automated Machine Learning Pipeline for tabular data.
         Designed for predictive maintenance applications, failure identification, failure prediction, condition
@@ -149,11 +151,12 @@ class Pipeline:
         self.mainDir = 'AutoML/'
         self.target = re.sub('[^a-z0-9]', '_', target.lower())
         self.verbose = verbose
-        self.customFunction = custom_function
+        self.preprocessFunction = preprocess_function
         self.name = name
 
         # Checks
-        assert mode == 'regression' or mode == 'classification', 'Supported modes: regression, classification.'
+        if mode is not None:
+            assert mode == 'regression' or mode == 'classification', 'Supported modes: regression, classification.'
         assert max_lags < 50, 'Max_lags too big. Max 50.'
         assert 0 < information_threshold < 1, 'Information threshold needs to be within [0, 1'
         assert max_diff < 5, 'Max diff too big. Max 5.'
@@ -164,20 +167,16 @@ class Pipeline:
         if include_output and not sequence:
             warnings.warn('[AutoML] IMPORTANT: strongly advices to not include output without sequencing.')
 
-        # Objective
+        # Objective & Scorer
+        self.scorer = None
         if objective is not None:
-            self.objective = objective
-        else:
-            if mode == 'regression':
-                self.objective = 'neg_mean_squared_error'
-            elif mode == 'classification':
-                self.objective = 'accuracy'
-        assert isinstance(objective, str), 'Objective needs to be a string, not type {}'.format(type(objective))
-        assert objective in metrics.SCORERS.keys(), 'Metric not supported, look at sklearn.metrics.SCORERS.keys()'
-        self.scorer = metrics.SCORERS[self.objective]
+            assert isinstance(objective, str), 'Objective needs to be a string, not type {}'.format(type(objective))
+            assert objective in metrics.SCORERS.keys(), 'Metric not supported, look at sklearn.metrics.SCORERS.keys()'
+            self.scorer = metrics.SCORERS[objective]
 
         # Pipeline Params
         self.mode = mode
+        self.objective = objective
         self.version = version
         self.includeOutput = include_output
         self.plotEDA = plot_eda
@@ -185,7 +184,8 @@ class Pipeline:
         self.documentResults = document_results
 
         # Data Processing params
-        self.numCols = [] if num_cols is None else num_cols
+        self.intCols = [] if int_cols is None else int_cols
+        self.floatCols = [] if float_cols is None else float_cols
         self.dateCols = [] if date_cols is None else date_cols
         self.catCols = [] if cat_cols is None else cat_cols
         self.missingValues = missing_values
@@ -233,24 +233,20 @@ class Pipeline:
         # Flags
         self._set_flags()
 
-        # Create Directories
-        self._create_dirs()
+        # Directories
+        if not no_dirs:
+            print('exec')
+            # Create Directories
+            self._create_dirs()
 
-        # Load Version
-        self._load_version()
+            # Load Version
+            self._load_version()
 
         # Required sub-classes
-        self.dataProcesser = DataProcesser(target=self.target, num_cols=self.numCols, date_cols=self.dateCols,
-                                           cat_cols=self.catCols, missing_values=self.missingValues,
-                                           outlier_removal=self.outlierRemoval, z_score_threshold=self.zScoreThreshold,
-                                           folder=self.mainDir + 'Data/')
-        self.dataSampler = DataSampler(method='both', margin=0.1, cv_splits=self.cvSplits, shuffle=self.shuffle,
-                                       fast_run=False, objective=self.objective)
-        self.dataSequencer = Sequencer(back=self.sequenceBack, forward=self.sequenceForward,
-                                       shift=self.sequenceShift, diff=self.sequenceDiff)
-        self.featureProcesser = FeatureProcesser(mode=self.mode, max_lags=self.maxLags, max_diff=self.maxDiff,
-                                                 extract_features=self.extractFeatures, timeout=self.featureTimeout,
-                                                 information_threshold=self.informationThreshold)
+        self.dataSampler = DataSampler()
+        self.dataProcesser = DataProcesser()
+        self.dataSequencer = Sequencer()
+        self.featureProcesser = FeatureProcesser()
 
         # Store Pipeline Settings
         args = locals()
@@ -493,8 +489,11 @@ class Pipeline:
         assert self.target != '', "Empty target string."
         assert self.target in Utils.clean_keys(data).keys(), 'Target missing in data'
 
+        # Detect mode (classification / regression)
+        self._mode_detector(data)
+
         # Run Exploratory Data Analysis
-        self._eda()
+        self._eda(data)
 
         # Preprocess Data
         self._data_processing(data)
@@ -526,10 +525,35 @@ class Pipeline:
         self.is_fitted = True
         print('[AutoML] All done :)')
 
-    def _eda(self):
+    def _mode_detector(self, data: pd.DataFrame):
+        """
+        Detects the mode (Regression / Classification)
+        :param data: Data to detect mode on
+        """
+        # Only run if mode is not provided
+        if self.mode is None:
+
+            # Classification if string
+            if data[self.target].dtype == 'string':
+                self.mode = 'classification'
+                self.objective = 'neg_log_loss'
+
+            # Or if unique values is less than 10%
+            elif data[self.target].nunique() < 0.1 * len(data):
+                self.mode = 'classification'
+                self.objective = 'neg_log_loss'
+
+            # Else regression
+            else:
+                self.mode = 'regression'
+                self.objective = 'neg_mean_absolute_error'
+            self.scorer = metrics.SCORERS[self.objective]
+        return
+
+    def _eda(self, data: pd.DataFrame):
         if self.plotEDA:
             print('[AutoML] Starting Exploratory Data Analysis')
-            eda = DataExplorer(self.x, y=self.y,
+            eda = DataExplorer(data.drop(self.target, axis=1), y=data[self.target],
                                mode=self.mode,
                                folder=self.mainDir,
                                version=self.version)
@@ -540,6 +564,11 @@ class Pipeline:
         Organises the data cleaning. Heavy lifting is done in self.dataProcesser, but settings etc. needs
         to be organised.
         """
+        self.dataProcesser = DataProcesser(target=self.target, int_cols=self.intCols, float_cols=self.floatCols,
+                                           date_cols=self.dateCols, cat_cols=self.catCols,
+                                           missing_values=self.missingValues,
+                                           outlier_removal=self.outlierRemoval, z_score_threshold=self.zScoreThreshold)
+
         if os.path.exists(self.mainDir + 'Data/Cleaned_v{}.csv'.format(self.version)):
             print('[AutoML] Loading Cleaned Data')
 
@@ -569,8 +598,10 @@ class Pipeline:
         # If no columns were provided, load them from data processor
         if self.dateCols is None:
             self.dateCols = self.settings['data_processing']['date_cols']
-        if self.numCols is None:
-            self.dateCols = self.settings['data_processing']['num_cols']
+        if self.intCols is None:
+            self.dateCols = self.settings['data_processing']['int_cols']
+        if self.floatCols is None:
+            self.floatCols = self.settings['data_processing']['float_cols']
         if self.catCols is None:
             self.catCols = self.settings['data_processing']['cat_cols']
 
@@ -594,6 +625,8 @@ class Pipeline:
         Does not guarantee to return balanced classes. (Methods are data dependent)
         """
         # Only necessary for classification
+        self.dataSampler = DataSampler(method='both', margin=0.1, cv_splits=self.cvSplits, shuffle=self.shuffle,
+                                       fast_run=False, objective=self.objective)
         if self.mode == 'classification':
             # Check if exists
             if os.path.exists(self.mainDir + 'Data/Balanced_v{}.csv'.format(self.version)):
@@ -621,6 +654,8 @@ class Pipeline:
         Sequences the data. Useful mostly for problems where older samples play a role in future values.
         The settings of this module are NOT AUTOMATIC
         """
+        self.dataSequencer = Sequencer(back=self.sequenceBack, forward=self.sequenceForward,
+                                       shift=self.sequenceShift, diff=self.sequenceDiff)
         if self.sequence:
             if os.path.exists(self.mainDir + 'Data/Sequence_v{}.csv'.format(self.version)):
                 print('[AutoML] Loading Sequenced Data')
@@ -648,6 +683,9 @@ class Pipeline:
         Organises feature processing. Heavy lifting is done in self.featureProcesser, but settings, etc.
         needs to be organised.
         """
+        self.featureProcesser = FeatureProcesser(mode=self.mode, max_lags=self.maxLags, max_diff=self.maxDiff,
+                                                 extract_features=self.extractFeatures, timeout=self.featureTimeout,
+                                                 information_threshold=self.informationThreshold)
         # Check if exists
         if os.path.exists(self.mainDir + 'Data/Extracted_v{}.csv'.format(self.version)):
             print('[AutoML] Loading Extracted Features')
@@ -1120,10 +1158,6 @@ class Pipeline:
 
         return self
 
-    def _error_analysis(self):
-        # todo implement
-        pass
-
     def convert_data(self, x: pd.DataFrame) -> [pd.DataFrame, pd.Series]:
         """
         Function that uses the same process as the pipeline to clean data.
@@ -1173,8 +1207,8 @@ class Pipeline:
             print('[AutoML] Predicting with {}, v{}'.format(type(self.bestModel).__name__, self.version))
 
         # Custom code
-        if self.customFunction is not None:
-            exec(self.customFunction)
+        if self.preprocessFunction is not None:
+            exec(self.preprocessFunction)
 
         # Convert
         x, y = self.convert_data(data)
@@ -1202,8 +1236,8 @@ class Pipeline:
             type(self.bestModel).__name__)
 
         # Custom code
-        if self.customFunction is not None:
-            exec(self.customFunction)
+        if self.preprocessFunction is not None:
+            exec(self.preprocessFunction)
 
         # Print
         if self.verbose > 0:
